@@ -3,32 +3,26 @@ import random
 import logging
 import uvloop
 import sys
+import signal
 
-from aiokafka import AIOKafkaConsumer, AIOKafkaProducer, ConsumerRecord, TopicPartition
+from aiokafka import AIOKafkaConsumer, AIOKafkaProducer
 from typing import (
-    Awaitable,
-    AsyncGenerator,
     Callable,
-    Dict,
     List,
     Optional,
-    Tuple,
-    Type,
-    NamedTuple,
-    Union,
 )
-from nptyping import NDArray
 from contextlib import AsyncExitStack
-from queue import Queue
 from functools import partial
 from multiprocessing import Process
 
-from metta.common import profiler, shared_memory, time_utils
+from proto_profiler import ProtoTimer
+from proto_profiler.trace_pb2 import Trace
+
+from metta.common import profiler, shared_memory
 from metta.common.shared_memory import SharedMemoryClient
-from metta.common.topics import ProtobufMessage, Topic, Message, MessageData, NewMessage
-from metta.common.topic_registry import TopicRegistry
-from metta.proto.trace_pb2 import Trace
-from metta.proto.topic_pb2 import DataLocation, TopicMessage
+from metta.topics.topics import Topic, Message, NewMessage
+from metta.topics.topic_registry import TopicRegistry
+from metta.types.topic_pb2 import DataLocation, TopicMessage
 
 
 class Node(AsyncExitStack):
@@ -197,12 +191,20 @@ class Node(AsyncExitStack):
             for parition, offset_and_ts in offsets.items():
                 self.consumer.seek(parition, offset_and_ts.offset)
 
+        profiler = ProtoTimer(disable=not profile)
+
         async for record in self.consumer():
-            input_msg = await self._parse(record.value)
-            output_msgs = await self._process(input_msg)
-            for output_msg in output_msgs:
-                await self._publish(output_msg)
-            if end_ts is not None and input_msg.timestamp >= end_ts:
+            try:
+                input_msg = await self._parse(record.value)
+                output_msgs = await self._process(input_msg)
+                for output_msg in output_msgs:
+                    await self._publish(output_msg)
+                    if profile:
+                        profiler.register(output_msg)
+                if end_ts is not None and input_msg.timestamp >= end_ts:
+                    break
+            except Exception as e:
+                logging.error(e)
                 break
 
     def register(self, forward_fn: Callable[[List[Message]], Optional[NewMessage]]):
@@ -215,16 +217,12 @@ class Node(AsyncExitStack):
         self,
         start_ts: Optional[int] = None,
         end_ts: Optional[int] = None,
+        seek_to_latest: bool = False,
         profile: Optional[bool] = False,
-    ) -> Process:
-        def _run():
-            async def exec():
-                async with self as node:
-                    await node.run(start_ts, end_ts, profile)
+    ):
+        async def _run():
+            async with self as node:
+                await node.run(start_ts, end_ts, seek_to_latest, profile)
 
-            uvloop.install()
-            asyncio.run(exec())
-
-        p = Process(target=_run)
-        p.start()
-        return p
+        uvloop.install()
+        asyncio.run(_run())
