@@ -1,15 +1,13 @@
 import asyncio
+import proto_profiler
 
 from typing import (
-    Awaitable,
     List,
     Optional,
 )
 
-from metta.common import profiler
-from metta.common.topics import Topic, NewMessage
+from metta.topics.topics import Topic, NewMessage
 from metta.nodes.node import Node
-from metta.proto.topic_pb2 import DataLocation
 
 
 class SourceNode(Node):
@@ -37,8 +35,8 @@ class SourceNode(Node):
             for output_msg in output_msgs:
                 trace = output_msg.trace
                 if trace is None:
-                    trace = profiler.init_trace()
-                profiler.touch_trace(trace, self.publish_topic)
+                    trace = proto_profiler.init_trace()
+                proto_profiler.touch_trace(trace, self.publish_topic)
                 output_msg.trace = trace
         return output_msgs
 
@@ -50,7 +48,11 @@ class SourceNode(Node):
         profile: Optional[bool] = False,
     ) -> None:
 
+        profiler = proto_profiler.ProtoTimer(disable=not profile)
+
         while True:
             output_msgs = await self._process()
             for output_msg in output_msgs:
-                await self._publish(output_msg)
\ No newline at end of file
+                await self._publish(output_msg)
+                if profile:
+                    profiler.register(output_msg)
\ No newline at end of file
